@@ -23,7 +23,6 @@
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 #include "test.h"
-__FBSDID("$FreeBSD: src/usr.bin/cpio/test/test_basic.c,v 1.4 2008/08/25 06:39:29 kientzle Exp $");
 
 static void
 verify_files(const char *msg)
@@ -230,6 +229,8 @@ DEFINE_TEST(test_basic)
 	basic_cpio("copy_odc", "--format=odc", "", msg, msg);
 	basic_cpio("copy_newc", "-H newc", "", result, "2 blocks\n");
 	basic_cpio("copy_cpio", "-H odc", "", msg, msg);
+	msg = "1 block\n";
+	basic_cpio("copy_bin", "-H bin", "", msg, msg);
 	msg = canSymlink() ? "9 blocks\n" : "8 blocks\n";
 	basic_cpio("copy_ustar", "-H ustar", "", msg, msg);
 
