@@ -25,7 +25,6 @@
  */
 
 #include "archive_platform.h"
-__FBSDID("$FreeBSD: head/lib/libarchive/archive_entry.c 201096 2009-12-28 02:41:27Z kientzle $");
 
 #ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
@@ -372,6 +371,12 @@ archive_entry_filetype(struct archive_entry *entry)
 	return (AE_IFMT & entry->acl.mode);
 }
 
+int
+archive_entry_filetype_is_set(struct archive_entry *entry)
+{
+	return (entry->ae_set & AE_SET_FILETYPE);
+}
+
 void
 archive_entry_fflags(struct archive_entry *entry,
     unsigned long *set, unsigned long *clear)
@@ -425,6 +430,12 @@ archive_entry_gid(struct archive_entry *entry)
 	return (entry->ae_stat.aest_gid);
 }
 
+int
+archive_entry_gid_is_set(struct archive_entry *entry)
+{
+	return (entry->ae_set & AE_SET_GID);
+}
+
 const char *
 archive_entry_gname(struct archive_entry *entry)
 {
@@ -631,6 +642,12 @@ archive_entry_perm(struct archive_entry *entry)
 	return (~AE_IFMT & entry->acl.mode);
 }
 
+int
+archive_entry_perm_is_set(struct archive_entry *entry)
+{
+	return (entry->ae_set & AE_SET_PERM);
+}
+
 dev_t
 archive_entry_rdev(struct archive_entry *entry)
 {
@@ -759,6 +776,12 @@ archive_entry_uid(struct archive_entry *entry)
 	return (entry->ae_stat.aest_uid);
 }
 
+int
+archive_entry_uid_is_set(struct archive_entry *entry)
+{
+	return (entry->ae_set & AE_SET_UID);
+}
+
 const char *
 archive_entry_uname(struct archive_entry *entry)
 {
@@ -827,6 +850,7 @@ archive_entry_set_filetype(struct archive_entry *entry, unsigned int type)
 	entry->stat_valid = 0;
 	entry->acl.mode &= ~AE_IFMT;
 	entry->acl.mode |= AE_IFMT & type;
+	entry->ae_set |= AE_SET_FILETYPE;
 }
 
 void
@@ -861,6 +885,7 @@ archive_entry_set_gid(struct archive_entry *entry, la_int64_t g)
 {
 	entry->stat_valid = 0;
 	entry->ae_stat.aest_gid = g;
+	entry->ae_set |= AE_SET_GID;
 }
 
 void
@@ -1145,6 +1170,7 @@ archive_entry_set_mode(struct archive_entry *entry, mode_t m)
 {
 	entry->stat_valid = 0;
 	entry->acl.mode = m;
+	entry->ae_set |= AE_SET_PERM | AE_SET_FILETYPE;
 }
 
 void
@@ -1220,6 +1246,7 @@ archive_entry_set_perm(struct archive_entry *entry, mode_t p)
 	entry->stat_valid = 0;
 	entry->acl.mode &= AE_IFMT;
 	entry->acl.mode |= ~AE_IFMT & p;
+	entry->ae_set |= AE_SET_PERM;
 }
 
 void
@@ -1354,6 +1381,7 @@ archive_entry_set_uid(struct archive_entry *entry, la_int64_t u)
 {
 	entry->stat_valid = 0;
 	entry->ae_stat.aest_uid = u;
+	entry->ae_set |= AE_SET_UID;
 }
 
 void
