@@ -25,7 +25,6 @@
  */
 
 #include "archive_platform.h"
-__FBSDID("$FreeBSD: head/lib/libarchive/archive_write_set_format_shar.c 189438 2009-03-06 05:58:56Z kientzle $");
 
 #ifdef HAVE_ERRNO_H
 #include <errno.h>
@@ -42,6 +41,7 @@ __FBSDID("$FreeBSD: head/lib/libarchive/archive_write_set_format_shar.c 189438 2
 #include "archive_entry.h"
 #include "archive_private.h"
 #include "archive_write_private.h"
+#include "archive_write_set_format_private.h"
 
 struct shar {
 	int			 dump;
@@ -59,8 +59,8 @@ struct shar {
 	struct archive_string	 quoted_name;
 };
 
-static int	archive_write_shar_finish(struct archive_write *);
-static int	archive_write_shar_destroy(struct archive_write *);
+static int	archive_write_shar_close(struct archive_write *);
+static int	archive_write_shar_free(struct archive_write *);
 static int	archive_write_shar_header(struct archive_write *,
 		    struct archive_entry *);
 static ssize_t	archive_write_shar_data_sed(struct archive_write *,
@@ -106,25 +106,25 @@ archive_write_set_format_shar(struct archive *_a)
 	struct archive_write *a = (struct archive_write *)_a;
 	struct shar *shar;
 
-	/* If someone else was already registered, unregister them. */
-	if (a->format_destroy != NULL)
-		(a->format_destroy)(a);
+	archive_check_magic(_a, ARCHIVE_WRITE_MAGIC,
+	    ARCHIVE_STATE_NEW, "archive_write_set_format_shar");
 
-	shar = (struct shar *)malloc(sizeof(*shar));
+	/* If someone else was already registered, unregister them. */
+	if (a->format_free != NULL)
+		(a->format_free)(a);
+
+	shar = (struct shar *)calloc(1, sizeof(*shar));
 	if (shar == NULL) {
 		archive_set_error(&a->archive, ENOMEM, "Can't allocate shar data");
 		return (ARCHIVE_FATAL);
 	}
-	memset(shar, 0, sizeof(*shar));
 	archive_string_init(&shar->work);
 	archive_string_init(&shar->quoted_name);
 	a->format_data = shar;
-
-	a->pad_uncompressed = 0;
 	a->format_name = "shar";
 	a->format_write_header = archive_write_shar_header;
-	a->format_finish = archive_write_shar_finish;
-	a->format_destroy = archive_write_shar_destroy;
+	a->format_close = archive_write_shar_close;
+	a->format_free = archive_write_shar_free;
 	a->format_write_data = archive_write_shar_data_sed;
 	a->format_finish_entry = archive_write_shar_finish_entry;
 	a->archive.archive_format = ARCHIVE_FORMAT_SHAR_BASE;
@@ -169,8 +169,7 @@ archive_write_shar_header(struct archive_write *a, struct archive_entry *entry)
 	}
 
 	/* Save the entry for the closing. */
-	if (shar->entry)
-		archive_entry_free(shar->entry);
+	archive_entry_free(shar->entry);
 	shar->entry = archive_entry_clone(entry);
 	name = archive_entry_pathname(entry);
 
@@ -195,8 +194,8 @@ archive_write_shar_header(struct archive_write *a, struct archive_entry *entry)
 		archive_entry_set_size(entry, 0);
 		if (archive_entry_hardlink(entry) == NULL &&
 		    archive_entry_symlink(entry) == NULL) {
-			archive_set_error(&a->archive, ARCHIVE_ERRNO_MISC,
-			    "shar format cannot archive this");
+			__archive_write_entry_filetype_unsupported(
+			    &a->archive, entry, "shar");
 			return (ARCHIVE_WARN);
 		}
 	}
@@ -266,12 +265,12 @@ archive_write_shar_header(struct archive_write *a, struct archive_entry *entry)
 				    shar->quoted_name.s, shar->quoted_name.s);
 			} else {
 				if (shar->dump) {
+					unsigned int mode = archive_entry_mode(entry) & 0777;
 					archive_string_sprintf(&shar->work,
 					    "uudecode -p > %s << 'SHAR_END'\n",
 					    shar->quoted_name.s);
 					archive_string_sprintf(&shar->work,
-					    "begin %o ",
-					    archive_entry_mode(entry) & 0777);
+					    "begin %o ", mode);
 					shar_quote(&shar->work, name, 0);
 					archive_strcat(&shar->work, "\n");
 				} else {
@@ -289,8 +288,7 @@ archive_write_shar_header(struct archive_write *a, struct archive_entry *entry)
 			    "mkdir -p %s > /dev/null 2>&1\n",
 			    shar->quoted_name.s);
 			/* Record that we just created this directory. */
-			if (shar->last_dir != NULL)
-				free(shar->last_dir);
+			free(shar->last_dir);
 
 			shar->last_dir = strdup(name);
 			/* Trim a trailing '/'. */
@@ -308,15 +306,15 @@ archive_write_shar_header(struct archive_write *a, struct archive_entry *entry)
 			break;
 		case AE_IFCHR:
 			archive_string_sprintf(&shar->work,
-			    "mknod %s c %d %d\n", shar->quoted_name.s,
-			    archive_entry_rdevmajor(entry),
-			    archive_entry_rdevminor(entry));
+			    "mknod %s c %ju %ju\n", shar->quoted_name.s,
+			    (uintmax_t)archive_entry_rdevmajor(entry),
+			    (uintmax_t)archive_entry_rdevminor(entry));
 			break;
 		case AE_IFBLK:
 			archive_string_sprintf(&shar->work,
-			    "mknod %s b %d %d\n", shar->quoted_name.s,
-			    archive_entry_rdevmajor(entry),
-			    archive_entry_rdevminor(entry));
+			    "mknod %s b %ju %ju\n", shar->quoted_name.s,
+			    (uintmax_t)archive_entry_rdevmajor(entry),
+			    (uintmax_t)archive_entry_rdevminor(entry));
 			break;
 		default:
 			return (ARCHIVE_WARN);
@@ -349,11 +347,13 @@ archive_write_shar_data_sed(struct archive_write *a, const void *buff, size_t n)
 	 * twice before entering the loop, so make sure three additional
 	 * bytes can be written.
 	 */
-	if (archive_string_ensure(&shar->work, ensured + 3) == NULL)
-		__archive_errx(1, "Out of memory");
+	if (archive_string_ensure(&shar->work, ensured + 3) == NULL) {
+		archive_set_error(&a->archive, ENOMEM, "Out of memory");
+		return (ARCHIVE_FATAL);
+	}
 
 	if (shar->work.length > ensured) {
-		ret = (*a->compressor.write)(a, shar->work.s,
+		ret = __archive_write_output(a, shar->work.s,
 		    shar->work.length);
 		if (ret != ARCHIVE_OK)
 			return (ARCHIVE_FATAL);
@@ -377,7 +377,7 @@ archive_write_shar_data_sed(struct archive_write *a, const void *buff, size_t n)
 
 		if (buf >= buf_end) {
 			shar->work.length = buf - shar->work.s;
-			ret = (*a->compressor.write)(a, shar->work.s,
+			ret = __archive_write_output(a, shar->work.s,
 			    shar->work.length);
 			if (ret != ARCHIVE_OK)
 				return (ARCHIVE_FATAL);
@@ -406,16 +406,18 @@ uuencode_group(const char _in[3], char out[4])
 	out[3] = UUENC( 0x3f & t );
 }
 
-static void
-uuencode_line(struct shar *shar, const char *inbuf, size_t len)
+static int
+_uuencode_line(struct archive_write *a, struct shar *shar, const char *inbuf, size_t len)
 {
-	char tmp_buf[3], *buf;
+	char *buf;
 	size_t alloc_len;
 
 	/* len <= 45 -> expanded to 60 + len byte + new line */
 	alloc_len = shar->work.length + 62;
-	if (archive_string_ensure(&shar->work, alloc_len) == NULL)
-		__archive_errx(1, "Out of memory");
+	if (archive_string_ensure(&shar->work, alloc_len) == NULL) {
+		archive_set_error(&a->archive, ENOMEM, "Out of memory");
+		return (ARCHIVE_FATAL);
+	}
 
 	buf = shar->work.s + shar->work.length;
 	*buf++ = UUENC(len);
@@ -426,21 +428,33 @@ uuencode_line(struct shar *shar, const char *inbuf, size_t len)
 		buf += 4;
 	}
 	if (len != 0) {
+		char tmp_buf[3];
 		tmp_buf[0] = inbuf[0];
 		if (len == 1)
 			tmp_buf[1] = '\0';
 		else
 			tmp_buf[1] = inbuf[1];
 		tmp_buf[2] = '\0';
-		uuencode_group(inbuf, buf);
+		uuencode_group(tmp_buf, buf);
 		buf += 4;
 	}
 	*buf++ = '\n';
-	if ((buf - shar->work.s) > (ptrdiff_t)(shar->work.length + 62))
-		__archive_errx(1, "Buffer overflow");
+	if ((buf - shar->work.s) > (ptrdiff_t)(shar->work.length + 62)) {
+		archive_set_error(&a->archive,
+		    ARCHIVE_ERRNO_MISC, "Buffer overflow");
+		return (ARCHIVE_FATAL);
+	}
 	shar->work.length = buf - shar->work.s;
+	return (ARCHIVE_OK);
 }
 
+#define uuencode_line(__a, __shar, __inbuf, __len) \
+	do { \
+		int r = _uuencode_line(__a, __shar, __inbuf, __len); \
+		if (r != ARCHIVE_OK) \
+			return (ARCHIVE_FATAL); \
+	} while (0)
+
 static ssize_t
 archive_write_shar_data_uuencode(struct archive_write *a, const void *buff,
     size_t length)
@@ -464,7 +478,7 @@ archive_write_shar_data_uuencode(struct archive_write *a, const void *buff,
 			shar->outpos += n;
 			return length;
 		}
-		uuencode_line(shar, shar->outbuff, 45);
+		uuencode_line(a, shar, shar->outbuff, 45);
 		src += n;
 		n = length - n;
 	} else {
@@ -472,13 +486,13 @@ archive_write_shar_data_uuencode(struct archive_write *a, const void *buff,
 	}
 
 	while (n >= 45) {
-		uuencode_line(shar, src, 45);
+		uuencode_line(a, shar, src, 45);
 		src += 45;
 		n -= 45;
 
 		if (shar->work.length < 65536)
 			continue;
-		ret = (*a->compressor.write)(a, shar->work.s,
+		ret = __archive_write_output(a, shar->work.s,
 		    shar->work.length);
 		if (ret != ARCHIVE_OK)
 			return (ARCHIVE_FATAL);
@@ -506,7 +520,7 @@ archive_write_shar_finish_entry(struct archive_write *a)
 		/* Finish uuencoded data. */
 		if (shar->has_data) {
 			if (shar->outpos > 0)
-				uuencode_line(shar, shar->outbuff,
+				uuencode_line(a, shar, shar->outbuff,
 				    shar->outpos);
 			archive_strcat(&shar->work, "`\nend\n");
 			archive_strcat(&shar->work, "SHAR_END\n");
@@ -517,7 +531,7 @@ archive_write_shar_finish_entry(struct archive_write *a)
 		 * directories; defer that to end of script.
 		 */
 		archive_string_sprintf(&shar->work, "chmod %o ",
-		    archive_entry_mode(shar->entry) & 07777);
+		    (unsigned int)(archive_entry_mode(shar->entry) & 07777));
 		shar_quote(&shar->work, archive_entry_pathname(shar->entry), 1);
 		archive_strcat(&shar->work, "\n");
 
@@ -531,6 +545,7 @@ archive_write_shar_finish_entry(struct archive_write *a)
 				archive_strcat(&shar->work, ":");
 				shar_quote(&shar->work, g, 1);
 			}
+			archive_strcat(&shar->work, " ");
 			shar_quote(&shar->work,
 			    archive_entry_pathname(shar->entry), 1);
 			archive_strcat(&shar->work, "\n");
@@ -560,7 +575,7 @@ archive_write_shar_finish_entry(struct archive_write *a)
 	if (shar->work.length < 65536)
 		return (ARCHIVE_OK);
 
-	ret = (*a->compressor.write)(a, shar->work.s, shar->work.length);
+	ret = __archive_write_output(a, shar->work.s, shar->work.length);
 	if (ret != ARCHIVE_OK)
 		return (ARCHIVE_FATAL);
 	archive_string_empty(&shar->work);
@@ -569,7 +584,7 @@ archive_write_shar_finish_entry(struct archive_write *a)
 }
 
 static int
-archive_write_shar_finish(struct archive_write *a)
+archive_write_shar_close(struct archive_write *a)
 {
 	struct shar *shar;
 	int ret;
@@ -592,7 +607,7 @@ archive_write_shar_finish(struct archive_write *a)
 
 	archive_strcat(&shar->work, "exit\n");
 
-	ret = (*a->compressor.write)(a, shar->work.s, shar->work.length);
+	ret = __archive_write_output(a, shar->work.s, shar->work.length);
 	if (ret != ARCHIVE_OK)
 		return (ARCHIVE_FATAL);
 
@@ -607,7 +622,7 @@ archive_write_shar_finish(struct archive_write *a)
 }
 
 static int
-archive_write_shar_destroy(struct archive_write *a)
+archive_write_shar_free(struct archive_write *a)
 {
 	struct shar *shar;
 
